;;; CONTOURS.LSP - a lisp program by Tony Hotchkiss
;;; Finds the mid-boundary between two polylines
;;; using parametric distances around the polyines
;;; June 2005
(defun err (s)
  (if (= s "Function cancelled")
    (princ "\nCONTOURS - cancelled: ")
    (progn (princ "\nCONTOURS - Error: ")
	   (princ s)
	   (terpri)
    ) ;_ progn
  ) ;_ end of if
  (resetting)
  (princ "SYSTEM VARIABLES have been reset\n")
  (princ)
) ;_ end of err
(defun setv (systvar newval)
  (setq x (read (strcat systvar "1")))
  (set x (getvar systvar))
  (setvar systvar newval)
) ;_ end of setv
(defun setting ()
  (setq oerr *error*)
  (setq *error* err)
  (setv "CMDECHO" 0)
  (setv "BLIPMODE" 0)
  (setv "OSMODE" 0)
  (setv "LUPREC" 8)
) ;_ end of setting
(defun rsetv (systvar)
  (setq x (read (strcat systvar "1")))
  (setvar systvar (eval x))
) ;_ end of rsetv
(defun resetting ()
  (rsetv "CMDECHO")
  (rsetv "BLIPMODE")
  (rsetv "OSMODE")
  (rsetv "LUPREC")
  (setq *error* oerr)
) ;_ end of resetting

(defun contour ()
  (vl-load-com)
  (setq	*thisdrawing* (vla-get-activedocument
			(vlax-get-acad-object)
		      ) ;_ end of vla-get-activedocument
	*modelspace*  (vla-get-ModelSpace *thisdrawing*)
  ) ;_ end of setq
  (setq plineobjs (get-plines))
  (initget 6)
  (setq numpoints (getint "\nPrecision <100>: "))
  (if (null numpoints)
    (setq numpoints 100)
  ) ;_ end of if
  (contours numpoints plineobjs)
;;;  (contours2 numpoints plineobjs)
) ;_ end-of contour

(defun get-plines ()
  (setq	pl-list	nil
	obj	nil
	pline	"AcDbPolyline"
	2dpline "AcDb2dPolyline"
  ) ;_ end of setq
  (setq	ssets	  (vla-get-selectionsets *thisdrawing*)
	ssetcount (vla-get-count ssets)
  ) ;_ end of setq
  (if (> ssetcount 0)
    (repeat ssetcount
      (vla-delete (vla-item ssets 0))
    ) ;_ end of repeat
  ) ;_ end of if
  (if (= (vla-get-count ssets) 0)
    (setq ssobj (vla-add ssets "ss1"))
  ) ;_ end of if
  (vla-clear ssobj)
  (setq no-ent 1)
  (while no-ent
    (prompt "\nSelect 2 Polylines: ")
    (vla-Selectonscreen ssobj)
    (if	(> (vla-get-count ssobj) 0)
      (progn
	(setq no-ent nil)
	(setq i (- 1))
	(repeat	(vla-get-count ssobj)
	  (setq
	    obj	(vla-item ssobj
			  (vlax-make-variant (setq i (1+ i)))
		) ;_ end of vla-item
	  ) ;_ end of setq
	  (cond
	    ((or (= (vla-get-Objectname  obj) pline)
		 (= (vla-get-Objectname obj) 2dpline)
	     ) ;_ end of or
	     (setq pl-list
		    (append pl-list (list obj))
	     ) ;_ end of setq
	    )
	  ) ;_ end-of cond
	) ;_ end of repeat
      ) ;_ end of progn
      (prompt "\nNo entities selected, try again.")
    ) ;_ end of if
    (if	(and (= nil no-ent) (< (length pl-list) 2))
      (progn
	(alert "2 Polylines not selected")
	(quit)
      ) ;_ end of progn
    ) ;_ end of if
  ) ;_ end of while  
  (vla-delete (vla-item ssets 0))
  pl-list
) ;_ end of get-plines

(defun contours (prec plobjs)
  (setq	pl1	  (car plobjs)
	pl2	  (cadr plobjs)
	lyr	  (vla-get-Layer pl1)
	start-pl1 (vlax-curve-getStartParam pl1)
	end-pl1	  (vlax-curve-getEndParam pl1)
	start-pl2 (vlax-curve-getStartParam pl2)
	end-pl2	  (vlax-curve-getEndParam pl2)
	param1	  (/ (- end-pl1 start-pl1) prec)
	param2	  (/ (- end-pl2 start-pl2) prec)
	i	  0
	2Dpoint-list	  nil
  ) ;_ end of setq  
  (repeat (1+ prec)
    (setq p1 (vlax-curve-getPointAtParam pl1 (* i param1)))
    (setq p2 (vlax-curve-getPointAtParam pl2 (* i param2)))
    (if	(and (/= p1 nil) (/= p2 nil))
      (progn
	(setq dist (distance p1 p2)
	      ang  (angle p1 p2)
	      p3   (polar p1 ang (/ dist 2))
	) ;_ end of setq
	(setq 2Dp	   (list (car p3) (cadr p3))
	      2Dpoint-list (append 2Dpoint-list 2Dp)
	) ;_ end of setq
      ) ;_ end of progn
    ) ;_ end of if
    (setq i (1+ i))
  ) ;_ end of repeat
  (setq	arraySpace
	 (vlax-make-safearray
	   vlax-vbdouble
	   (cons 0
		 (- (length 2Dpoint-list) 1)
	   ) ; array dimension
	 ) ;_ end of vlax-make-safearray
  ) ;_ end of setq
  (setq	vertex-array
	 (vlax-safearray-fill arraySpace 2Dpoint-list)
  ) ;_ end of setq
  (vlax-make-variant vertex-array)
  (setq obj (vla-AddLightWeightPolyline *modelspace* vertex-array))
  (vla-put-Layer obj lyr)
) ;_ end of contours

(defun contours2 (prec plobjs)
  (setq	pl1	  (car plobjs)
	pl2	  (cadr plobjs)
	lyr	  (vla-get-Layer pl1)
	start-pl1 (vlax-curve-getStartParam pl1)
	end-pl1	  (vlax-curve-getEndParam pl1)
	start-pl2 (vlax-curve-getStartParam pl2)
	end-pl2	  (vlax-curve-getEndParam pl2)
	param1	  (/ (- end-pl1 start-pl1) prec)
	param2	  (/ (- end-pl2 start-pl2) prec)
	i	  0
	2Dpoint-list	  nil
  ) ;_ end of setq  
  (repeat (1+ prec)
    (setq p1 (vlax-curve-getPointAtParam pl1 (* i param1)))
    (if	(/= p1 nil)
      (setq p2 (vlax-curve-getClosestPointTo pl2 p1))
      (setq p2 nil)
    ) ;_ end of if
;;;    (if	(and (/= p1 nil) (/= p2 nil))
;;;      (progn
	(if (not (or (= p1 nil) (= p2 nil)))
	  (progn
	    (setq dist (distance p1 p2)
		  ang  (angle p1 p2)
		  p3   (polar p1 ang (/ dist 2))
	    ) ;_ end of setq
	    (setq 2Dp	       (list (car p3) (cadr p3))
		  2Dpoint-list (append 2Dpoint-list 2Dp)
	    ) ;_ end of setq
	  ) ;_ end of progn
	) ;_ end of if
;;;      ) ;_ end of progn
;;;    ) ;_ end of if
    (setq i (1+ i))
  ) ;_ end of repeat
  (setq	arraySpace
	 (vlax-make-safearray
	   vlax-vbdouble
	   (cons 0
		 (- (length 2Dpoint-list) 1)
	   ) ; array dimension
	 ) ;_ end of vlax-make-safearray
  ) ;_ end of setq
  (setq	vertex-array
	 (vlax-safearray-fill arraySpace 2Dpoint-list)
  ) ;_ end of setq
  (vlax-make-variant vertex-array)
  (setq obj (vla-AddLightWeightPolyline *modelspace* vertex-array))
  (vla-put-Layer obj "0")
) ;_ end of contours2

(defun contours3 (prec plobjs)
  (setq	pl1	  (car plobjs)
	pl2	  (cadr plobjs)
	lyr	  (vla-get-Layer pl1)
	start-pl1 (vlax-curve-getStartParam pl1)
	end-pl1	  (vlax-curve-getEndParam pl1)
	start-pl2 (vlax-curve-getStartParam pl2)
	end-pl2	  (vlax-curve-getEndParam pl2)
	param1	  (/ (- end-pl1 start-pl1) prec)
	param2	  (/ (- end-pl2 start-pl2) prec)
	i	  0
	2Dpoint-list	  nil
  ) ;_ end of setq  
  (repeat (1+ prec)
    (setq p1 (vlax-curve-getPointAtParam pl1 (* i param1)))
    (setq p2 (vlax-curve-getPointAtParam pl2 (* i param2)))
    (if	(and (/= p1 nil) (/= p2 nil))
      (progn
	(setq dist (distance p1 p2)
	      ang  (angle p1 p2)
	      p3   (polar p1 ang (/ dist 2))
	) ;_ end of setq
	(setq 2Dp	   (list (car p3) (cadr p3))
	      2Dpoint-list (append 2Dpoint-list 2Dp)
	) ;_ end of setq
      ) ;_ end of progn
    ) ;_ end of if
    (setq i (1+ i))
  ) ;_ end of repeat
  (setq	arraySpace
	 (vlax-make-safearray
	   vlax-vbdouble
	   (cons 0
		 (- (length 2Dpoint-list) 1)
	   ) ; array dimension
	 ) ;_ end of vlax-make-safearray
  ) ;_ end of setq
  (setq	vertex-array
	 (vlax-safearray-fill arraySpace 2Dpoint-list)
  ) ;_ end of setq
  (vlax-make-variant vertex-array)
  (setq obj (vla-AddLightWeightPolyline *modelspace* vertex-array))
  (vla-put-Layer obj lyr)
) ;_ end of contours3

(defun c:ct ()
  (setting)
  (contour)
  (resetting)
  (princ)
) ;_ end of c:sg

(prompt "\nCopyright (c) 2005, Tony Hotchkiss")
(prompt "\nEnter CT to start")
